@@ -1666,6 +1666,81 @@ smbinning.plot=function(ivout,option="dist",sub=""){
 }
 # End Plotting ################################################################
 
+# Ini: PSI 20170821 ###########################################################
+#' Population Stability Index
+#' 
+#' Often models are developed using multiple periods in time for a number of reasons.
+#' For example, to avoid seasonality, to increase the size of the population, and some others.
+#' With a metrics like the Population Stability Index (PSI), users can check if there is
+#' a significant variation in the distribution of a certain feature by partition (usually time)
+#' using the first one as the reference.
+#' @param df Data frame.
+#' @param y Column name the indicates the different partitions.
+#' @param x Feature to be evaluated in terms of stability (It must be factor).
+#' @return Three crosstabs by feature and period that show the frequency (psicnt), 
+#' percentage (psipct) and PSI (psimg), and a plot for the analyzed characteristic.
+#' @examples 
+#' # Load library and its dataset
+#' library(smbinning)
+#'
+#' # Check stability for income
+#' smbinning.psi(df=chileancredit,y="period",x="inc") 
+
+smbinning.psi = function(df,y,x){
+  i=which(names(df)==x)
+  j=which(names(df)==y)
+  if(!is.data.frame(df)){return(stop("Not a data frame"))}
+  else if(identical(i,integer(0))){return(stop(paste("Characteristic",x,"not found")))}
+  else if(identical(j,integer(0))){return(stop(paste("Characteristic",y,"not found")))}
+  else if(class(df[,i])!="factor"){return(stop("x must be formatted as factor"))}
+  else {
+    psicnt=table(df[,i],df[,j],useNA = "ifany") # Table with counts including NA
+    options(scipen=999) # Remove Scientific Notation
+    psipct=prop.table(psicnt, margin=2) # Table with column percentage
+    psimg=psipct # Shell for PSI table
+    n=ncol(psipct) # Number of columns (Periods)
+    m=nrow(psipct) # Number of rowa (Periods)
+    
+    psimg[,1]=0 # Step 1: PSI=0 for first column
+    
+    # PSI Period VS First Period
+    for (k in 2:n){
+      for (l in 1:m){
+        if(psipct[l,1]>0 & psipct[l,k]>0) 
+        {psimg[l,k]=round((psipct[l,k]-psipct[l,1])*log(psipct[l,k]/psipct[l,1]),8)}
+        else {psimg[l,k]=0}
+      }
+    }
+    
+    psimg=rbind(psimg, PSI=colSums(psimg))
+    psimg=as.table(psimg) # Table with Mg PSI
+    psitable=psimg[nrow(psimg),] # Extract total PSI only
+    psitable=as.data.frame(psitable)
+    # Plot
+    psitable$Partition=rownames(psitable) # Create column "Partition"
+    rownames(psitable)=NULL  # Remove rownames
+    names(psitable)=c("PSI","Partition") # Rename columns
+    psitable=psitable[,c("Partition","PSI")] # Reorder
+    
+    maxpsi=max(psitable$PSI)
+    psiylim=ifelse(maxpsi<0.25,0.25,maxpsi)
+    plot(psitable$PSI,
+         main = paste("Stability:",x),
+         xlab = paste("Partition:",y),
+         ylab="PSI",
+         ylim=c(0,psiylim), 
+         pch=19,
+         col="black",
+         xaxt = "n") # Remove x axis values
+    abline(h=0.1,lty=2)
+    abline(h=0.25,lty=2)
+    axis(1,at=1:nrow(psitable),psitable$Partition) # Add period
+    
+    list(psicnt=psicnt,psipct=psipct,psimg=psimg)  
+    }
+}
+
+# End: PSI 20170821 ###########################################################
 
 # Begin Model Scaling 20170821 ################################################
 #' Scaling
@@ -2250,9 +2325,10 @@ smbinning.sumiv.plot=function(sumivt, cex=0.9){
 #'   \item dd: Number of direct deposits per month.
 #'   \item online: Indicator of active online (Yes, No).
 #'   \item rnd: Random number to select testing and training samples.
+#'   \item period: Factor that indicates the year/month of the data.
 #'   }
 #'
-#' @format Data frame with 10,000 rows and 22 columns.
+#' @format Data frame with 10,000 rows and 23 columns.
 #' @name chileancredit
 NULL
 # End: Chilean Credit Data ####################################################
